using System;
using System.Runtime.InteropServices;

namespace Microsoft.RfidComWrapper
{
    /// <summary>
    /// An interface for the RFID operations wrapper.
    /// </summary>
    [ComVisible(true)]
    [Guid("C98BD7FF-8B59-484a-BA4D-B3EF1F52E402"), InterfaceType(ComInterfaceType.InterfaceIsDual)]
    public interface IRfidWrapper
    {
        /// <summary>
        /// Writes a tag to a device.
        /// </summary>
        /// <param name="deviceName">The RFID Device name to connect to.</param>
        /// <param name="tagId">The tag ID to write.</param>
        /// <param name="tagData">The tag data to write.</param>
        void WriteTag(string deviceName, string tagId, string tagData);
        
        /// <summary>
        /// Reads the tags count.
        /// </summary>
        /// <param name="deviceName">The name of the RFID device to read data from.</param>
        /// <returns>The tags count.</returns>
        int TagsCount(string deviceName);

        /// <summary>
        /// Reads the tag data.
        /// </summary>
        /// <param name="deviceName">The name of the RFID device to read data from.</param>
        /// <param name="index">The index of the tag to read.</param>
        /// <returns>The tag data or null if the tag cannot be read. Check <see cref="ErrorCode"/> and <see cref="ErrorMessage"/> properties for the error description if any.</returns>
        RfidTagWrapper ReadTag(string deviceName, int index);
        
        /// <summary>
        /// Gets the last error code.
        /// </summary>
        /// <value>An integer that represents the error code. 0 if no error is detected.</value>
        int ErrorCode
        {
            get;
        }

        /// <summary>
        /// Gets the last error message.
        /// </summary>
        /// <value>A string that represents the error description.</value>
        string ErrorMessage
        {
            get;
        }

        /// <summary>
        /// Starts tracking RFID events.
        /// </summary>
        /// <returns>True if the operation has succeeded. False if an error has occured. Check <see cref="ErrorMessage"/> property for details.</returns>
        bool StartEventsTracking();

        /// <summary>
        /// Stops tracking RFID events.
        /// </summary>
        /// <returns>True if the operation has succeeded. False if an error has occured. Check <see cref="ErrorMessage"/> property for details.</returns>
        bool StopEventsTracking();
    }
}
