using System;
using System.Collections.Generic;
using Microsoft.SensorServices.Rfid;

namespace Microsoft.RfidComWrapper
{
    /// <summary>
    /// A handler for RFID process events.
    /// </summary>
    [CLSCompliant(false)]
    public class RfidEventsHandler : RfidEventHandlerBase
    {
        public override void Init(Dictionary<string, object> parameters, RfidProcessContext container)
        {
        }

        static RfidEventHandlerMetadata RfidMetadata;
        /// <summary>
        /// Returns the metadata for parameters of this event handler.
        /// This method is called by the Process binding tool.
        /// </summary>
        /// <returns>Mappings from names to ParameterMetaData.</returns>
        public static RfidEventHandlerMetadata GetEventHandlerMetadata(bool vendorExtensionAlso)
        {
            if (RfidMetadata == null)
            {
                Dictionary<string, RfidEventHandlerParameterMetadata> metadata = new Dictionary<string, RfidEventHandlerParameterMetadata>();
                RfidMetadata = new RfidEventHandlerMetadata("RfidComWrapper", metadata);
            }
            return RfidMetadata;
        }

        /// <summary>
        /// A handler for the RFID read event. Called by the RFID process when the tag read event occurs.
        /// </summary>
        [RfidEventHandlerMethod]
        public TagReadEvent HandleTagReadEvent(TagReadEvent tagReadEvent)
        {
            using (RfidServicedWrapper service = new RfidServicedWrapper())
            {
                service.PlaceTag(RfidTagWrapper.CreateFromReadTagEvent(tagReadEvent));
            }
            
            return tagReadEvent;
        }
    }   
}
