using System;
using System.Collections.Generic;
using System.EnterpriseServices;
using System.Runtime.InteropServices;

namespace Microsoft.RfidComWrapper
{
    /// <summary>
    /// A COM+ component used as a singletone to share RFID events data.
    /// </summary>
    [ObjectPooling(1, 1)]
    [ComVisible(true)]
    public class RfidServicedWrapper : ServicedComponent
    {
        /// <summary>
        /// A queue with tags.
        /// </summary>
        private List<RfidTagWrapper> tagQueue = new List<RfidTagWrapper>();
        private bool tagTracking;

        protected override bool CanBePooled()
        {
            return true;
        }
        
        /// <summary>
        /// Starts tracking tags, e. g. starts placing tags to the tags queue.
        /// </summary>
        public void StartTagTracking()
        {
            tagTracking = true;
        }
        
        /// <summary>
        /// Stops tracking tags.
        /// </summary>
        public void StopTagTracking()
        {
            tagTracking = false;
        }
        
        /// <summary>
        /// Places a tag to the queue for further reading.
        /// </summary>
        public void PlaceTag(RfidTagWrapper tag)
        {
            if (tag != null && tagTracking)
            {
                lock (tagQueue)
                {
                    tagQueue.Add(tag);
                }
            }
        }

        /// <summary>
        /// Returns the first tag from the tags queue or null if the queue is empty.
        /// </summary>
        public RfidTagWrapper ReadTag()
        {
            lock (tagQueue)
            {
                if (tagQueue.Count > 0)
                {
                    RfidTagWrapper tag = tagQueue[tagQueue.Count - 1];
                    tagQueue.RemoveAt(tagQueue.Count - 1);
                    return tag;
                }
            }

            return null;
        }
    }
}
