using System;
using System.Runtime.InteropServices;
using System.Text;
using Microsoft.SensorServices.Rfid;

namespace Microsoft.RfidComWrapper
{
    /// <summary>
    /// Wrapper for the RFID Tag object.
    /// </summary>
    [ComVisible(true)]
    [Guid("3D994A62-FDFB-4535-93E3-F2A8311425AE"), ClassInterface(ClassInterfaceType.None)]
    [Serializable]
    public class RfidTagWrapper : IRfidTagWrapper
    {
        private DateTime time;
        private string source;
        private string deviceName;
        private string tagData;
        private string tagId;

        /// <summary>
        /// The tag ID.
        /// </summary>
        public string TagId
        {
            get { return tagId; }
        }

        /// <summary>
        /// The tag data.
        /// </summary>
        public string TagData
        {
            get { return tagData; }
        }

        /// <summary>
        /// The name of the device that received a tag.
        /// </summary>
        public string DeviceName
        {
            get { return deviceName; }
        }

        /// <summary>
        /// The tag source.
        /// </summary>
        public string Source
        {
            get { return source; }
        }

        /// <summary>
        /// The tag's creation time.
        /// </summary>
        public DateTime Time
        {
            get { return time; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RfidTagWrapper"/> class that is empty. 
        /// </summary>
        public RfidTagWrapper()
        {
            time = DateTime.Now;
            source = string.Empty;
            deviceName = string.Empty;
            tagData = string.Empty;
            tagId = string.Empty;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RfidTagWrapper"/> class with given property values. 
        /// </summary>
        /// <param name="time">The tag's creation time.</param>
        /// <param name="source">The tag source.</param>
        /// <param name="deviceName">The device name.</param>
        /// <param name="tagData">The tag data.</param>
        /// <param name="tagId">The tag ID.</param>
        public RfidTagWrapper(DateTime time, string source, string deviceName, string tagData, string tagId)
        {
            this.time = time;
            this.source = source;
            this.deviceName = deviceName;
            this.tagData = tagData;
            this.tagId = tagId;
        }

        /// <summary>
        /// Creates an <see cref="RfidTagWrapper"/> object from the data available in the tag read event.
        /// </summary>
        internal static RfidTagWrapper CreateFromReadTagEvent(TagReadEvent tagReadEvent)
        {
            if (tagReadEvent == null)
            {
                return null;
            }

            byte[] data = tagReadEvent.GetData();
            byte[] tagId = tagReadEvent.GetId();
            
            return new RfidTagWrapper(tagReadEvent.Time, tagReadEvent.Source, tagReadEvent.DeviceName,
                data == null ? string.Empty : ASCIIEncoding.ASCII.GetString(data),
                tagId == null ? string.Empty : ASCIIEncoding.ASCII.GetString(tagId));
        }
    }
}